@@ -15,11 +15,12 @@ with 'WWW::FBX::Role::Auth';
  
 use namespace::autoclean;
 
-our $VERSION="0.06";
+our $VERSION="0.07";
 
 has lwp_args    => ( isa => 'HashRef', is => 'ro', default => sub { {} } );
 has ua          => ( isa => 'LWP::UserAgent', is => 'rw', lazy => 1, builder => '_build_ua' );
 has uar         => ( isa => 'HashRef', is => 'rw' );
+has uarh        => ( isa => 'HTTP::Response', is => 'rw' );
 has [ qw/app_id app_name app_version device_name/ ] => ( 
     isa => 'Str', is => 'ro', required => 1 );
 
@@ -43,9 +44,12 @@ sub _json_request {
     my $msg = $self->_prepare_request($http_method, $uri, $args, $content_type);
     my $res = $self->_send_request($msg);
 
-    #Store response
+    #Store response content
     $self->uar( $self->_parse_result ($res, $args ) );
 
+    #And HTTP response RAW
+    $self->uarh( $res );
+
     return $self->uar;
 }
  
@@ -54,6 +58,7 @@ sub _prepare_request {
  
     my $msg;
  
+#    use Data::Dumper ; print "$http_method: $uri\n"; print "params:", Dumper $args;
     if( $http_method eq 'PUT' ) {
         $msg = PUT( $uri, Content => encode_json  $args  );
     }
@@ -66,7 +71,6 @@ sub _prepare_request {
             $msg = POST( $uri,  Content_Type => 'application/json', Content =>  encode_json $args );
         }
         elsif ( $content_type eq "form-data" ) {
-            print "$http_method : $uri\n";
             $msg = POST($uri, Content_Type => 'form-data', Content => [ map { ref $_ ? $_ : encode_utf8 $_ } %$args ]);
         }
         else {
@@ -76,10 +80,10 @@ sub _prepare_request {
     else {
         croak "unexpected HTTP method: $http_method";
     }
- 
+
     return $msg;
 }
- 
+
 sub _query_string_for {
     my ( $self, $args ) = @_;
 
@@ -92,12 +96,12 @@ sub _query_string_for {
 }
 
 sub _send_request { shift->ua->request(shift) }
- 
+
 sub _parse_result {
     my ($self, $res, $args) = @_;
- 
+
     my $content = $res->content;
- 
+
     my $j_obj = length $content ? try { $self->from_json($content) } : {};
 
     #Die if message contains an API error (even on HTTP 200)
@@ -110,12 +114,12 @@ sub _parse_result {
 
     #API Download file does not return JSON!!
     #If answer is 200 and not json, return unchanged (but still pack it in an HashRef for uar type check..)
-    return {content => $content} if $res->is_success;
+    return {filename => $res->filename, content => $content} if $res->filename and $res->is_success;
 
     #Else die on HTTP failures, which might contain a json response or not
     my $error = WWW::FBX::Error->new(http_response => $res);
     $error->fbx_error($j_obj) if ref $j_obj;
- 
+
     die $error;
 }
 
@@ -146,16 +150,16 @@ Authentication is provided through the Auth role but other authentication mechan
     use Scalar::Util 'blessed';
 
     my $res;
-    eval { 
-        my $fbx = WWW::FBX->new( 
-            app_id => "APP ID", 
-            app_name => "APP NAME", 
-            app_version => "1.0", 
-            device_name => "debian", 
-            track_id => "48", 
+    eval {
+        my $fbx = WWW::FBX->new(
+            app_id => "APP ID",
+            app_name => "APP NAME",
+            app_version => "1.0",
+            device_name => "debian",
+            track_id => "48",
             app_token => "2/g43EZYD8AO7tbnwwhmMxMuELtTCyQrV1goMgaepHWGrqWlloWmMRszCuiN2ftp",
         );
-        print "You are now authenticated with track_id ", $fbx->track_id, " and app_token ", $fbx->app_token, "\n"; 
+        print "You are now authenticated with track_id ", $fbx->track_id, " and app_token ", $fbx->app_token, "\n";
         print "App permissions are:\n";
         while ( my( $key, $value ) = each %{ $fbx->uar->{result}{permissions} } ) {
             print "\t $key\n" if $value;
@@ -169,7 +173,7 @@ Authentication is provided through the Auth role but other authentication mechan
 
     if ( my $err = $@ ) {
         die $@ unless blessed $err && $err->isa('WWW::FBX::Error');
- 
+
         warn "HTTP Response Code: ", $err->code, "\n",
              "HTTP Message......: ", $err->message, "\n",
              "API Error.........: ", $err->error, "\n",
@@ -192,10 +196,14 @@ The constructor takes care of detecting the API version and authentication.
 
 Return values are perl hashes.
 
-The same response is also available through the uar method.
+The hash response of the last request is available through the uar method.
+
+The HTTP::Response is available through the uarh method.
 
 This distribution is heavily inspired from L<Net::Twitter>.
 
+The currently available services implemented in this module is given in L<WWW::FBX::Role::API::APIv3>. Pull requests are welcome!
+
 =head1 LICENSE
 
 Copyright (C) Laurent Kislaire.
