@@ -16,52 +16,55 @@ my ($_api_url, $_base_url) = ( "", "http://mafreebox.free.fr" );
 sub api_url { $_api_url = $_[1]; }
 sub base_url { $_base_url = $_[1]; }
 
-sub fbx_api_method { 
+sub fbx_api_method {
     my $caller = shift;
     my $name   = shift;
     my %options = (
         @_,
     );
+    my $args;
 
     #Remove trailing _
     $name =~ s/_$//;
 
     my $class = Moose::Meta::Class->initialize($caller);
- 
-    my ($arg_names, $path) = @options{qw/required path/};
-    $arg_names = $options{params} if @$arg_names == 0 && @{$options{params}} == 1;
- 
+
+    my ($arg_names, $all_args, $path) = @options{qw/required params path/};
+
     my $code = sub {
         my $self = shift;
- 
-        # copy callers args since we may add ->{source}
         my $args = ref $_[-1] eq 'HASH' ? { %{pop @_} } : {};
- 
-        croak sprintf "$name expected %d args", scalar @$arg_names if @_ > @$arg_names;
- 
+
+        croak sprintf "$name expected %d args", scalar @$all_args if @_ > @$all_args;
+
         # promote positional args to named args
         for ( my $i = 0; @_; ++$i ) {
-            my $param = $arg_names->[$i];
+            my $param = $all_args->[$i];
             croak "duplicate param $param: both positional and named"
                 if exists $args->{$param};
- 
+
             $args->{$param} = shift;
         }
- 
-        # promote boolean parameters
-        for my $boolean_arg ( @{ $options{booleans} } ) {
-            if ( exists $args->{$boolean_arg} ) {
-                next if $args->{$boolean_arg} =~ /^true|false$/;
-                $args->{$boolean_arg} = $args->{$boolean_arg} ? 'true' : 'false';
+
+        #find unknown keys
+        for my $arg (keys %$args) {
+            unless ( grep { $_ eq $arg } @$all_args ) {
+                die "Unknown argument $arg for $name\n" , "Description:$options{description}" ,
+                    "Params:", join(",",@$all_args), "\nRequired:", join(",", @$arg_names), "\n" ;
             }
         }
- 
-        # replace placeholder arguments
-        my $local_path = $path;
-        $local_path =~ s/:(\w+)/delete $args->{$1} or croak "required arg '$1' missing"/eg;
-        $local_path .= delete $args->{suff} if exists $args->{suff};
 
-        my $uri = URI->new($_base_url . $_api_url . "/$local_path");
+        #find missing req
+        for my $req (@$arg_names) {
+            unless ( grep { $_ eq $req } keys %$args or !defined ($args->{req}) ) {
+                die "Missing required param $req for $name\n", "Description:$options{description}" ,
+                    "Params:", join(",",@$all_args), "\nRequired:", join(",", @$arg_names), "\n" ;
+            }
+        }
+
+        $path .= delete $args->{suff} if exists $args->{suff};
+
+        my $uri = URI->new("$_base_url$_api_url/$path");
 
         return $self->_json_request(
             $options{method},
@@ -87,23 +90,22 @@ package WWW::FBX::Meta::Method;
 use Moose;
 use Carp::Clan qw/^(?:WWW::FBX|Moose|Class::MOP)/;
 extends 'Moose::Meta::Method';
- 
+
 use namespace::autoclean;
- 
+
 has description     => ( isa => 'Str', is => 'ro', required => 1 );
 has path            => ( isa => 'Str', is => 'ro', required => 1 );
 has method          => ( isa => 'Str', is => 'ro', default => 'GET' );
 has params          => ( isa => 'ArrayRef[Str]', is => 'ro', default => sub { [] } );
 has required        => ( isa => 'ArrayRef[Str]', is => 'ro', default => sub { [] } );
 has returns         => ( isa => 'Str', is => 'ro', predicate => 'has_returns' );
-has booleans        => ( isa => 'ArrayRef[Str]', is => 'ro', default => sub { [] } );
 has content_type    => ( isa => 'Str', is => 'ro', default => '' );
 has suff            => ( isa => 'Str', is => 'ro', default => '' );
- 
+
 #Build hash where keys are attribute names
 my %valid_attribute_names = map { $_->init_arg => 1 }
                             __PACKAGE__->meta->get_all_attributes;
- 
+
 sub new {
     my $class = shift;
     my %args  = @_;
@@ -111,7 +113,7 @@ sub new {
     #Stack arguments that are not expected attributes
     my @invalid_attributes = grep { !$valid_attribute_names{$_} } keys %args;
     croak "unexpected argument(s): @invalid_attributes" if @invalid_attributes;
- 
+
     #Create method
     $class->SUPER::wrap(@_);
 }
